/*
    This file is part of web3.js.

    web3.js is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    web3.js is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with web3.js.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * @file bch.js
 * @author Marek Kotewicz <marek@ethdev.com>
 * @author Fabian Vogelsteller <fabian@ethdev.com>
 * @date 2015
 */

"use strict";

var formatters = require('../formatters');
var Method = require('../method');
var c = require('../../utils/config');
var Iban = require('../iban');
var transfer = require('../transfer');

function Bch(web3) {
    this._requestManager = web3._requestManager;

    var self = this;

    methods().forEach(function(method) {
        method.attachToObject(self);
        method.setRequestManager(self._requestManager);
    });

    this.iban = Iban;
    this.sendIBANTransaction = transfer.bind(null, this);
}

Object.defineProperty(Bch.prototype, 'defaultAccount', {
    get: function () {
        return c.defaultAccount;
    },
    set: function (val) {
        c.defaultAccount = val;
        return val;
    }
});

Object.defineProperty(Bch.prototype, 'defaultSlpAccount', {
    get: function () {
        return c.defaultSlpAccount;
    },
    set: function (val) {
        c.defaultSlpAccount = val;
        return val;
    }
});

var methods = function () {
    var getBalance = new Method({
        name: 'getBalance',
        call: 'bch_getBalance',
        params: 1,
    });

    var getTransaction = new Method({
        name: 'getTransaction',
        call: 'bch_getTransactionByHash',
        params: 1,
        outputFormatter: formatters.outputTransactionFormatter
    });

    var sendRawTransaction = new Method({
        name: 'sendRawTransaction',
        call: 'bch_sendRawTransaction',
        params: 1,
        inputFormatter: [null]
    });

    var sendTransaction = new Method({
        name: 'sendTransaction',
        call: 'eth_sendTransaction',
        params: 1,
    });

    var sign = new Method({
        name: 'sign',
        call: 'eth_sign',
        params: 2,
    });

    return [
        getBalance,
        getTransaction,
        sendRawTransaction,
        sendTransaction,
        sign
    ];
};

module.exports = Bch;
